
#include <stdio.h>
#include <hip/hip_runtime.h>

/* vector Assign A to B */
__global__ void vectorAssign(const float *A, float *B, int numElements)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;

    if (i < numElements)
    {
        B[i] = A[i];
    }
}

struct float3_m96
{
	    float x, y, z;
};

typedef struct float3_m96 float3_m96;

typedef float rvec3[3];
typedef float rvec4[4];

/* Host main routine */
int main(void)
{
	printf("sizeof float3:%lu\n", sizeof(float3));
	printf("sizeof float4:%lu\n", sizeof(float4));

	printf("sizeof rvec3:%lu\n", sizeof(rvec3));
	printf("sizeof rvec4:%lu\n", sizeof(rvec4));

	printf("sizeof float3_m96:%lu\n", sizeof(float3_m96));

	rvec3 h_r3A[2];
	float3 h_f3A[2];
	float3_m96 h_f3m96A[2];

	for(int i=0; i<2; i++)
	{
		h_r3A[i][0] = 0.0f;
		h_r3A[i][1] = 1.0f;
		h_r3A[i][2] = 2.0f;
	}

	for(int i=0; i<2; i++)
	{
		h_f3A[i].x = 3.0f;
		h_f3A[i].y = 4.0f;
		h_f3A[i].z = 5.0f;
	}

	for(int i=0; i<2; i++)
	{
		printf("%f\t%f\t%f\n", h_r3A[i][0], h_r3A[i][1], h_r3A[i][2]);
	}
	printf("\n");
	for(int i=0; i<2; i++)
	{
		printf("%f\t%f\t%f\n", h_f3A[i].x, h_f3A[i].y, h_f3A[i].z);
	}
	printf("\n");

    //Allocate the device input vector A
	int num=2;
	size_t size = num * sizeof(rvec3);

    rvec3 *d_r3A = NULL;
    hipMalloc((void **)&d_r3A, size);

    // Copy the host input vectors A and B in host memory to the device input vectors in device memory
    hipMemcpy(d_r3A, h_r3A, size, hipMemcpyHostToDevice);
	hipMemcpy(h_r3A, d_r3A, size, hipMemcpyDeviceToHost);

	for(int i=0; i<2; i++)
	{
		printf("%f\t%f\t%f\n", h_r3A[i][0], h_r3A[i][1], h_r3A[i][2]);
	}
	printf("\n");

	size = num * sizeof(float3);
	float3 *d_f3A = NULL;
	hipMalloc((void **)&d_f3A, size);
    hipMemcpy(d_f3A, h_f3A, size, hipMemcpyHostToDevice);
	hipMemcpy(h_f3A, d_f3A, size, hipMemcpyDeviceToHost);

	for(int i=0; i<2; i++)
	{
		printf("%f\t%f\t%f\n", h_f3A[i].x, h_f3A[i].y, h_f3A[i].z);
	}
	printf("\n");

	printf("Cross test.\n");

    hipMemcpy(d_f3A, h_r3A, size, hipMemcpyHostToDevice);
	hipMemcpy(h_f3A, d_f3A, size, hipMemcpyDeviceToHost);
	for(int i=0; i<2; i++)
	{
		printf("%f\t%f\t%f\n", h_f3A[i].x, h_f3A[i].y, h_f3A[i].z);
	}
	printf("\n");

/*
    hipMemcpy(d_f3A, h_f3A, size, hipMemcpyHostToDevice);
	hipMemcpy(h_r3A, d_f3A, size, hipMemcpyDeviceToHost);
	for(int i=0; i<2; i++)
	{
		printf("%f\t%f\t%f\n", h_r3A[i][0], h_r3A[i][1], h_r3A[i][2]);
	}
	printf("\n");
*/
/*
	size = num * sizeof(float3_m96);
	float3 *d_f3m96A = NULL;
	hipMalloc((void **)&d_f3m96A, size);

    hipMemcpy(d_f3m96A, h_r3A, size, hipMemcpyHostToDevice);
	hipMemcpy(h_f3m96A, d_f3m96A, size, hipMemcpyDeviceToHost);
	for(int i=0; i<2; i++)
	{
		printf("%f\t%f\t%f\n", h_f3m96A[i].x, h_f3m96A[i].y, h_f3m96A[i].z);
	}
	printf("\n");
*/

    // Launch the Vector Add CUDA Kernel
    //int threadsPerBlock = 256;
    //int blocksPerGrid =(numElements + threadsPerBlock - 1) / threadsPerBlock;
    //printf("CUDA kernel launch with %d blocks of %d threads\n", blocksPerGrid, threadsPerBlock);
    //hipLaunchKernelGGL((vectorAssign), dim3(blocksPerGrid), dim3(threadsPerBlock), 0, 0, d_A, d_B, numElements);

    // Free device global memory
    hipFree(d_r3A);

    return 0;
}
