
#include <stdio.h>
#include "hip/hip_runtime_api.h"
#include "hip/hip_vector_types.h"
#include "hipfft.h"

#define NX 1000
#define BATCH 1

/* Host main routine */
int main(void)
{
    hipfftHandle plan;

    // Print the vector length to be used, and compute its size
    int numElements = NX;
    size_t size = numElements * sizeof(hipfftComplex);

    // Allocate the host
    hipfftComplex *h_A = (hipfftComplex *)malloc(size);
    hipfftComplex *h_B = (hipfftComplex *)malloc(size);

    // Initialize the host input vectors
    for (int i = 0; i < numElements; ++i)
    {
        h_A[i].x = rand()/(float)RAND_MAX;
        h_A[i].y = rand()/(float)RAND_MAX;
    }

    for(int i=0; i<numElements; i++)
    {
        printf("%f\t%f\n", h_A[i].x, h_A[i].y);
    }
    printf("\n");

    // Allocate the device
    hipfftComplex *d_A = NULL;
    hipMalloc((void **)&d_A, size);

    // Copy the host input vectors A and B in host memory to the device input vectors in device memory
    hipMemcpy(d_A, h_A, size, hipMemcpyHostToDevice);

    hipfftPlan1d(&plan, numElements, HIPFFT_C2C, BATCH);

    hipfftExecC2C(plan, d_A, d_A, HIPFFT_FORWARD);
    hipfftExecC2C(plan, d_A, d_A, HIPFFT_BACKWARD);

    hipDeviceSynchronize();

    // Copy the device result vector in device memory to the host result vector in host memory.
    hipMemcpy(h_B, d_A, size, hipMemcpyDeviceToHost);
    for(int i=0; i<numElements; i++)
    {
        printf("%f\t%f\n", h_B[i].x, h_B[i].y);
    }

    hipfftDestroy(plan);
    // Free device global memory
    hipFree(d_A);

    // Free host memory
    free(h_A);
    free(h_B);

    printf("Done\n");
    return 0;
}

