
#include <stdio.h>
#include "hip/hip_runtime_api.h"
#include "hip/hip_vector_types.h"
#include "rocfft.h"

#define NX 1000
#define BATCH 1

/* Host main routine */
int main(void)
{
	rocfft_setup();

    // Print the vector length to be used, and compute its size
    int numElements = NX;
    size_t size = numElements * sizeof(float2);

    // Allocate the host
    float2 *h_A = (float2 *)malloc(size);
    float2 *h_B = (float2 *)malloc(size);

    // Initialize the host input vectors
    for (int i = 0; i < numElements; ++i)
    {
        h_A[i].x = rand()/(float)RAND_MAX;
        h_A[i].y = rand()/(float)RAND_MAX;
    }

    for(int i=0; i<numElements; i++)
    {
        printf("%f\t%f\n", h_A[i].x, h_A[i].y);
    }
    printf("\n");

    // Allocate the device
    float2 *d_A = NULL;
    hipMalloc((void **)&d_A, size);

    // Copy the host input vectors A and B in host memory to the device input vectors in device memory
    hipMemcpy(d_A, h_A, size, hipMemcpyHostToDevice);

	rocfft_plan plan1 = NULL;
	rocfft_plan plan2 = NULL;
	size_t length = numElements;
	rocfft_plan_create(&plan1, rocfft_placement_inplace, rocfft_transform_type_complex_forward, rocfft_precision_single, 1, &length, 1, NULL);
	rocfft_plan_create(&plan2, rocfft_placement_inplace, rocfft_transform_type_complex_inverse, rocfft_precision_single, 1, &length, 1, NULL);
	rocfft_execute(plan1, (void**) &d_A, NULL, NULL);
	rocfft_execute(plan2, (void**) &d_A, NULL, NULL);

    hipDeviceSynchronize();

    // Copy the device result vector in device memory to the host result vector in host memory.
    hipMemcpy(h_B, d_A, size, hipMemcpyDeviceToHost);
    for(int i=0; i<numElements; i++)
    {
        printf("%f\t%f\n", h_B[i].x, h_B[i].y);
    }

	rocfft_plan_destroy(plan1);
	rocfft_plan_destroy(plan2);

    // Free device global memory
    hipFree(d_A);

	rocfft_cleanup();

    // Free host memory
    free(h_A);
    free(h_B);

    printf("Done\n");
    return 0;
}

