#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>

#include <hip/hip_runtime.h>
#include <mpi.h>

#define hipErrorCheck(a) do { \
    if (hipSuccess != (a)) { \
		fprintf(stderr, "HIP runtime error in line %d of file %s \
			: %s \n", __LINE__, __FILE__, hipGetErrorString(a) ); \
		exit(EXIT_FAILURE); \
	} \
} while(0);

int main(int argc, char **argv)
{
    int myid, numprocs;
	char myhostname[1024];
	float *s_buf_h, *r_buf_h;
	float *s_buf_d, *r_buf_d;

	int devsrc = atoi(argv[1]);
	int devdst = atoi(argv[2]);

    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &myid);
    MPI_Comm_size(MPI_COMM_WORLD, &numprocs);

	gethostname(myhostname, 1024);
	printf("Process: %d, On host: %s, devsrc: %d, devdst: %d\n", myid, myhostname, devsrc, devdst);

	//hip set device
	if(myid == 0) {
		hipErrorCheck(hipSetDevice(0));
	}
	else
	{
		hipErrorCheck(hipSetDevice(1));
		//hipSetDevice(devdst);
	}

	//init buffer
	//size_t numElements = 143;
	size_t numElements = (1<<30)/4;
	//printf("numElements: %lu\n", numElements);
	size_t size = numElements * sizeof(float);

	if(myid == 0) {
		s_buf_h = (float *)malloc(size);
		for (int i = 0; i < numElements; ++i)
		{   
			s_buf_h[i] = rand()/(float)RAND_MAX;
		}
		for (int i = 0; i < 10; ++i)
		{   
			printf("%f\n", s_buf_h[i]);
		}
		
		printf("numElements: %lu\n", numElements);
		printf("size: %lu\n", size);

		hipErrorCheck(hipMalloc((void**)&s_buf_d, size));
		//sleep(30);
		//hipErrorCheck(hipMemcpy(s_buf_d, s_buf_h, size, hipMemcpyHostToDevice));
		hipMemcpy(s_buf_d, s_buf_h, size, hipMemcpyHostToDevice);
	}
	//else {
//		r_buf_h = (float *)malloc(size);
//		hipErrorCheck(hipMalloc((void**)r_buf_d, size));
//	}

/*
	//send data
	MPI_Status status;
	int tag = 0;
	if(myid == 1) {
		MPI_Recv(r_buf_d, numElements, MPI_FLOAT, 0, tag, MPI_COMM_WORLD, &status);
	}
	else if(myid == 0) {
		MPI_Send(s_buf_d, numElements, MPI_FLOAT, 1, tag, MPI_COMM_WORLD);
	}
	MPI_Barrier(MPI_COMM_WORLD);

	//output buffer
	if(myid == 0) {
		printf("Process: %d, On host: %s\n", myid, myhostname);
		hipMemcpy(s_buf_h, s_buf_d, size, hipMemcpyDeviceToHost);
		printf("s_buf_h:\n");
		for(int i=0; i<10; ++i) {
			printf("%f\n", s_buf_h[i]);
		}
		printf("\n");
	}
	else {
		printf("Process: %d, On host: %s\n", myid, myhostname);
		hipMemcpy(r_buf_h, r_buf_d, size, hipMemcpyDeviceToHost);
		printf("r_buf_h:\n");
		for(int i=0; i<10; ++i) {
			printf("%f\n", r_buf_h[i]);
		}
		printf("\n");
	}
*/

    if(myid == 0) {
		hipFree(s_buf_d);
		free(s_buf_h);
	}

    if(myid == 0) {
		printf("Done\n");
        fflush(stdout);
    }

    MPI_Finalize();
    return 0;
}
