
#include <stdio.h>
#include <hip/hip_runtime.h>

/* vector Assign A to B */
__global__ void vectorAssign(const float *A, float *B, int numElements)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;

    if (i < numElements)
    {
        B[i] = A[i];
    }
}

/* Host main routine */
int main(void)
{
    // Print the vector length to be used, and compute its size
    int numElements = 10;
    size_t size = numElements * sizeof(float);

    // Allocate the host
    float *h_A = (float *)malloc(size);
    float *h_B = (float *)malloc(size);

    // Initialize the host input vectors
    for (int i = 0; i < numElements; ++i)
    {
        h_A[i] = rand()/(float)RAND_MAX;
    }

    for(int i=0; i<numElements; i++)
	{
		printf("%f\n", h_A[i]);
	}
	printf("\n");

    for(int i=0; i<numElements; i++)
	{
		printf("%f\n", h_B[i]);
	}
	printf("\n");

    // Allocate the device input vector A
    float *d_A = NULL;
    hipMalloc((void **)&d_A, size);
    float *d_B = NULL;
    hipMalloc((void **)&d_B, size);

    // Copy the host input vectors A and B in host memory to the device input vectors in device memory
    hipMemcpy(d_A, h_A, size, hipMemcpyHostToDevice);

    // Launch the Vector Add CUDA Kernel
    int threadsPerBlock = 256;
    int blocksPerGrid =(numElements + threadsPerBlock - 1) / threadsPerBlock;
    //printf("CUDA kernel launch with %d blocks of %d threads\n", blocksPerGrid, threadsPerBlock);
    hipLaunchKernelGGL((vectorAssign), dim3(blocksPerGrid), dim3(threadsPerBlock), 0, 0, d_A, d_B, numElements);

    // Copy the device result vector in device memory to the host result vector in host memory.
    //printf("Copy output data from the CUDA device to the host memory\n");
    hipMemcpy(h_B, d_B, size, hipMemcpyDeviceToHost);

    for(int i=0; i<numElements; i++)
	{
		printf("%f\n", h_B[i]);
	}

    // Free device global memory
    hipFree(d_A);
    hipFree(d_B);

    // Free host memory
    free(h_A);
    free(h_B);

    printf("Done\n");
    return 0;
}
